@@ -8,11 +8,15 @@ oauthlib.oauth2.draft_25
 This module is an implementation of various logic needed
 for signing and checking OAuth 2.0 draft 25 requests.
 """
+from oauthlib.common import Request
+from oauthlib.oauth2.draft25 import errors
 from .tokens import prepare_bearer_uri, prepare_bearer_headers
 from .tokens import prepare_bearer_body, prepare_mac_header
+from .tokens import BearerToken
 from .parameters import prepare_grant_uri, prepare_token_request
 from .parameters import parse_authorization_code_response
 from .parameters import parse_implicit_response, parse_token_response
+from .utils import params_from_uri
 
 
 AUTH_HEADER = 'auth_header'
@@ -32,6 +36,8 @@ class Client(object):
             token_type='Bearer',
             access_token=None,
             refresh_token=None,
+            mac_key=None,
+            mac_algorithm=None,
             **kwargs):
         """Initialize a client with commonly used attributes."""
 
@@ -40,6 +46,8 @@ class Client(object):
         self.token_type = token_type
         self.access_token = access_token
         self.refresh_token = refresh_token
+        self.mac_key = mac_key
+        self.mac_algorithm = mac_algorithm
 
     @property
     def token_types(self):
@@ -58,7 +66,7 @@ class Client(object):
         }
 
     def add_token(self, uri, http_method='GET', body=None, headers=None,
-            token_placement=None):
+            token_placement=None, **kwargs):
         """Add token to the request uri, body or authorization header.
 
         The access token type provides the client with the information
@@ -97,9 +105,9 @@ class Client(object):
             raise ValueError("Missing access token.")
 
         return self.token_types[self.token_type](uri, http_method, body,
-                    headers, token_placement)
+                    headers, token_placement, **kwargs)
 
-    def prepare_refresh_body(self, body='', refresh_token=None, scope=None):
+    def prepare_refresh_body(self, body='', refresh_token=None, scope=None, **kwargs):
         """Prepare an access token request, using a refresh token.
 
         If the authorization server issued a refresh token to the client, the
@@ -120,7 +128,7 @@ class Client(object):
         """
         refresh_token = refresh_token or self.refresh_token
         return prepare_token_request('refresh_token', body=body, scope=scope,
-                refresh_token=refresh_token)
+                refresh_token=refresh_token, **kwargs)
 
     def _add_bearer_token(self, uri, http_method='GET', body=None,
             headers=None, token_placement=None):
@@ -139,14 +147,14 @@ class Client(object):
         return uri, headers, body
 
     def _add_mac_token(self, uri, http_method='GET', body=None,
-            headers=None, token_placement=AUTH_HEADER):
+            headers=None, token_placement=AUTH_HEADER, ext=None, **kwargs):
         """Add a MAC token to the request authorization header.
 
         Warning: MAC token support is experimental as the spec is not yet stable.
         """
-        headers = prepare_mac_header(self.access_token, uri, self.key, http_method,
-                        headers=headers, body=body, ext=self.ext,
-                        hash_algorithm=self.hash_algorithm)
+        headers = prepare_mac_header(self.access_token, uri, self.mac_key, http_method,
+                        headers=headers, body=body, ext=ext,
+                        hash_algorithm=self.mac_algorithm, **kwargs)
         return uri, headers, body
 
     def _populate_attributes(self, response):
@@ -167,6 +175,12 @@ class Client(object):
         if 'code' in response:
             self.code = response.get('code')
 
+        if 'mac_key' in response:
+            self.mac_key = response.get('mac_key')
+
+        if 'mac_algorithm' in response:
+            self.mac_algorithm = response.get('mac_algorithm')
+
     def prepare_request_uri(self, *args, **kwargs):
         """Abstract method used to create request URIs."""
         raise NotImplementedError("Must be implemented by inheriting classes.")
@@ -520,10 +534,154 @@ class PasswordCredentialsClient(Client):
         self._populate_attributes(response)
         return response
 
-class AuthorizationServer(object):
-    def client_redirect_uris(self, client_identifier):
-        raise NotImplementedError("Must be implemented by inheriting classes.")
 
-    def redirect_uri(client_identifier, redirect_uri=None):
-        raise NotImplementedError("TODO")
+class AuthorizationEndpoint(object):
+    """Authorization endpoint - used by the client to obtain authorization
+    from the resource owner via user-agent redirection.
 
+    The authorization endpoint is used to interact with the resource
+    owner and obtain an authorization grant.  The authorization server
+    MUST first verify the identity of the resource owner.  The way in
+    which the authorization server authenticates the resource owner (e.g.
+    username and password login, session cookies) is beyond the scope of
+    this specification.
+
+    The endpoint URI MAY include an "application/x-www-form-urlencoded"
+    formatted (per Appendix B) query component ([RFC3986] section 3.4),
+    which MUST be retained when adding additional query parameters.  The
+    endpoint URI MUST NOT include a fragment component.
+
+    Since requests to the authorization endpoint result in user
+    authentication and the transmission of clear-text credentials (in the
+    HTTP response), the authorization server MUST require the use of TLS
+    as described in Section 1.6 when sending requests to the
+    authorization endpoint.
+
+    The authorization server MUST support the use of the HTTP "GET"
+    method [RFC2616] for the authorization endpoint, and MAY support the
+    use of the "POST" method as well.
+
+    Parameters sent without a value MUST be treated as if they were
+    omitted from the request.  The authorization server MUST ignore
+    unrecognized request parameters.  Request and response parameters
+    MUST NOT be included more than once.
+    """
+
+    def __init__(self, default_token=None, response_types=None):
+        self._response_types = response_types or {}
+        self._default_token = default_token or BearerToken()
+
+    @property
+    def response_types(self):
+        return self._response_types
+
+    @property
+    def default_token(self):
+        return self._default_token
+
+    def create_authorization_response(self, uri, http_method='GET', body=None,
+            headers=None):
+        """Extract response_type and route to the designated handler."""
+        request = Request(uri, http_method=http_method, body=body, headers=headers)
+        query_params = params_from_uri(self.request.uri)
+        body_params = self.request.decoded_body
+
+        # Prioritize response_type defined as query param over those in body.
+        # Chosen because the two core grant types utilizing the response type
+        # parameter both supply it in the uri. However it is not specified
+        # explicitely in RFC 6748.
+        if 'response_type' in query_params:
+            request.response_type = query_params.get('response_type')
+        elif 'response_type' in body_params:
+            request.response_type = body_params.get('response_type')
+        else:
+            raise errors.InvalidRequestError(
+                description='The response_type parameter is missing.')
+
+        if not request.response_type in self.response_types:
+            raise errors.UnsupportedResponseTypeError(
+                description='Invalid response type')
+
+        return self.response_types.get(
+                request.response_type).create_authorization_response(
+                        request, self.default_token)
+
+
+class TokenEndpoint(object):
+
+    def __init__(self, default_token=None, grant_types=None):
+        self._grant_types = grant_types or {}
+        self._default_token = default_token or BearerToken()
+
+    @property
+    def grant_types(self):
+        return self._grant_types
+
+    @property
+    def default_token(self):
+        return self._default_token
+
+    def create_token_response(self, uri, http_method='GET', body=None, headers=None):
+        """Extract grant_type and route to the designated handler."""
+        request = Request(uri, http_method=http_method, body=body, headers=headers)
+        query_params = params_from_uri(self.request.uri)
+        body_params = self.request.decoded_body
+
+        # Prioritize grant_type defined as body param over those in uri.
+        # Chosen because all three core grant types supply this parameter
+        # in the body. However it is not specified explicitely in RFC 6748.
+        if 'grant_type' in body_params:
+            request.grant_type = query_params.get('grant_type')
+        elif 'grant_type' in query_params:
+            request.grant_type = body_params.get('grant_type')
+        else:
+            raise errors.InvalidRequestError(
+                description='The grant_type parameter is missing.')
+
+        if not request.grant_type in self.grant_types:
+            raise errors.UnsupportedGrantTypeError(
+                description='Invalid response type')
+
+        return self.grant_types.get(
+                request.grant_type).create_token_response(
+                        request, self.default_token)
+
+
+class ResourceEndpoint(object):
+
+    def __init__(self, tokens=None):
+        self._tokens = tokens or {'Bearer': BearerToken()}
+
+    @property
+    def tokens(self):
+        return self._tokens
+
+    def verify_request(self, uri, http_method='GET', body=None, headers=None):
+        """Validate client, code etc, return body + headers"""
+        request = Request(uri, http_method, body, headers)
+        request.token_type = self.find_token_type(request)
+
+        # TODO(ib-lundgren): How to return errors is not strictly defined and
+        # should allow for customization.
+        if not request.token_type:
+            raise ValueError('Could not determine the token type.')
+
+        if not request.token_type in self.tokens:
+            raise ValueError('Unsupported token type.')
+
+        return self.tokens.get(request.token_type).validate_request(request)
+
+    def find_token_type(self, request):
+        """Token type identification.
+
+        RFC 6749 does not provide a method for easily differentiating between
+        different token types during protected resource access. We estimate
+        the most likely token type (if any) by asking each known token type
+        to give an estimation based on the request.
+        """
+        estimates = sorted((t.estimate_type(request) for t in self.tokens))
+        return estimates[0] if len(estimates) else None
+
+
+class Server(AuthorizationEndpoint, TokenEndpoint, ResourceEndpoint):
+    pass
